@@ -366,10 +366,14 @@ static gboolean key_press_event(GtkWidget * unused, GdkEventKey * event)
 
     case GDK_KP_Space:
     case GDK_space:
+    case GDK_Page_Down:
+    case GDK_KP_Page_Down:
         load_direction(1);
         break;
 
     case GDK_BackSpace:
+    case GDK_Page_Up:
+    case GDK_KP_Page_Up:
         load_direction(-1);
         break;
 
