@@ -77,6 +77,8 @@
  * are '\0' terminated strings representing decimal numbers.
  */
 
+#include <unistd.h>             /* isatty() */
+#include <sys/types.h>          /* size_t, off_t */
 #include <sys/mman.h>           /* mmap(), PROT_READ, MAP_PRIVATE, ... */
 #include <stdio.h>              /* FILE, fopen(), fread(), fwrite(), ... */
 #include <string.h>             /* strlen(), strrchr() */
@@ -100,25 +102,37 @@
 #include "decompression.h"
 #include "callbacks.h"
 #include "windows.h"
+#include "options.h"
 
 /* Max filename length in the progress window. */
 #define MAX_DISPLAYED_NAME_LENGTH 30
 
+extern rt_struct *rt;
+extern options_struct *options;
 extern GlivImage *current_image;
 
-static gchar *get_collection_source(void)
+static gchar *get_filename(const gchar * label, gboolean save)
 {
+    GtkFileChooserAction action;
+    const gchar *stock_button;
     GtkFileChooser *chooser;
     gint response;
-    const gchar *label = _("Choose a collection to load");
     gchar *filename = NULL;
 
+    if (save) {
+        action = GTK_FILE_CHOOSER_ACTION_SAVE;
+        stock_button = GTK_STOCK_SAVE;
+    } else {
+        action = GTK_FILE_CHOOSER_ACTION_OPEN;
+        stock_button = GTK_STOCK_OPEN;
+    }
+
     chooser = GTK_FILE_CHOOSER(gtk_file_chooser_dialog_new(label,
-                                                           get_current_window(),
-                                                           GTK_FILE_CHOOSER_ACTION_OPEN,
+                                                           NULL,
+                                                           action,
                                                            GTK_STOCK_CANCEL,
                                                            GTK_RESPONSE_CANCEL,
-                                                           GTK_STOCK_OPEN,
+                                                           stock_button,
                                                            GTK_RESPONSE_ACCEPT,
                                                            NULL));
 
@@ -133,42 +147,27 @@ static gchar *get_collection_source(void)
 
 static gchar *get_collection_destination(void)
 {
-    GtkFileChooser *chooser;
-    gint response;
-    const gchar *label = _("Choose a file to save the collection");
-    gchar *filename = NULL;
+    gchar *filename;
 
-    chooser = GTK_FILE_CHOOSER(gtk_file_chooser_dialog_new(label,
-                                                           get_current_window(),
-                                                           GTK_FILE_CHOOSER_ACTION_SAVE,
-                                                           GTK_STOCK_CANCEL,
-                                                           GTK_RESPONSE_CANCEL,
-                                                           GTK_STOCK_SAVE,
-                                                           GTK_RESPONSE_ACCEPT,
-                                                           NULL));
+    filename = get_filename(_("Choose a file to save the collection"), TRUE);
+    if (filename != NULL && g_file_test(filename, G_FILE_TEST_EXISTS)) {
+        GtkMessageDialog *overwrite =
+            GTK_MESSAGE_DIALOG(gtk_message_dialog_new(NULL,
+                                                      GTK_DIALOG_MODAL,
+                                                      GTK_MESSAGE_QUESTION,
+                                                      GTK_BUTTONS_YES_NO,
+                                                      _("Overwrite \"%s\" ?"),
+                                                      filename));
 
-    response = run_modal_dialog(GTK_DIALOG(chooser));
-
-    if (response == GTK_RESPONSE_ACCEPT) {
-        filename = gtk_file_chooser_get_filename(chooser);
-        if (g_file_test(filename, G_FILE_TEST_EXISTS)) {
-            GtkMessageDialog *overwrite =
-                GTK_MESSAGE_DIALOG(gtk_message_dialog_new
-                                   (get_current_window(), GTK_DIALOG_MODAL,
-                                    GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
-                                    _("Overwrite \"%s\" ?"), filename));
-
-	    response = run_modal_dialog(GTK_DIALOG(overwrite));
-	    if (response != GTK_RESPONSE_YES) {
-		g_free(filename);
-		filename = NULL;
-	    }
-
-	    gtk_widget_destroy(GTK_WIDGET(overwrite));
+        gint response = run_modal_dialog(GTK_DIALOG(overwrite));
+        if (response != GTK_RESPONSE_YES) {
+            g_free(filename);
+            filename = NULL;
         }
+
+        gtk_widget_destroy(GTK_WIDGET(overwrite));
     }
 
-    gtk_widget_destroy(GTK_WIDGET(chooser));
     return filename;
 }
 
@@ -180,6 +179,7 @@ static GtkProgressBar *progress;
 static GtkWindow *progress_window;
 static GtkLabel *elapsed_label, *total_label, *remaining_label;
 static time_t start_time;
+static gboolean is_saving;      /* loading or saving ? */
 
 static gboolean set_true(gboolean * ptr)
 {
@@ -207,8 +207,7 @@ static GtkLabel *add_time_display(GtkTable * table, const gchar * text,
     return time_label;
 }
 
-static void create_progress_dialog(const gchar * filename, gboolean serialize,
-                                   gpointer data)
+static void create_progress_dialog(const gchar * filename, gpointer cancel_data)
 {
     GtkButton *cancel;
     GtkTable *table;
@@ -225,12 +224,8 @@ static void create_progress_dialog(const gchar * filename, gboolean serialize,
     gtk_widget_show(GTK_WIDGET(progress));
 
     cancel = GTK_BUTTON(gtk_button_new_from_stock(GTK_STOCK_CANCEL));
-
-    if (serialize)
-        g_signal_connect(cancel, "clicked",
-                         G_CALLBACK(cancel_using_tree), data);
-    else
-        g_signal_connect_swapped(cancel, "clicked", G_CALLBACK(set_true), data);
+    g_signal_connect_swapped(cancel, "clicked", G_CALLBACK(set_true),
+                             cancel_data);
 
     gtk_widget_show(GTK_WIDGET(cancel));
 
@@ -248,21 +243,18 @@ static void create_progress_dialog(const gchar * filename, gboolean serialize,
     gtk_widget_show(GTK_WIDGET(table));
 
     progress_window = GTK_WINDOW(gtk_window_new(GTK_WINDOW_TOPLEVEL));
+    g_signal_connect_swapped(progress_window, "delete-event",
+                             G_CALLBACK(set_true), cancel_data);
 
-    if (serialize)
-        g_signal_connect(progress_window, "delete-event",
-                         G_CALLBACK(cancel_using_tree), NULL);
-    else
-        g_signal_connect_swapped(progress_window, "delete-event",
-                                 G_CALLBACK(set_true), data);
-
-    if (serialize) {
+    if (is_saving)
         title = g_strdup_printf(_("Saving collection: %s"),
                                 filename_to_utf8(filename));
-        gtk_window_set_title(progress_window, title);
-        g_free(title);
-    } else
-        gtk_window_set_title(progress_window, _("Loading GLiv collection"));
+    else
+        title = g_strdup_printf(_("Loading collection: %s"),
+                                filename_to_utf8(filename));
+
+    gtk_window_set_title(progress_window, title);
+    g_free(title);
 
     gtk_container_add(GTK_CONTAINER(progress_window), GTK_WIDGET(table));
     gtk_widget_show(GTK_WIDGET(progress_window));
@@ -296,39 +288,90 @@ static void update_times(time_t elapsed_time,
     g_free(text);
 }
 
+static gboolean need_refresh_progress(gboolean last_time)
+{
+    static GTimeVal last = { 0, 0 };
+    GTimeVal now;
+    glong usec_diff;
+
+    if (last_time) {
+        last.tv_sec = 0;
+        last.tv_usec = 0;
+        return TRUE;
+    }
+
+    if (last.tv_sec == 0 && last.tv_usec == 0) {
+        g_get_current_time(&last);
+        return TRUE;
+    }
+
+    g_get_current_time(&now);
+    usec_diff = (now.tv_sec - last.tv_sec) * G_USEC_PER_SEC +
+        now.tv_usec - last.tv_usec;
+
+    if (usec_diff < G_USEC_PER_SEC / 10)
+        /* Lower the redrawing load. */
+        return FALSE;
+
+    last = now;
+    return TRUE;
+}
+
+static FILE *get_terminal_output(void)
+{
+    static FILE *print;
+    static gboolean print_ok = FALSE;
+
+    if (!print_ok) {
+        if (isatty(fileno(stdout)))
+            print = stdout;
+
+        else if (isatty(fileno(stderr)))
+            print = stderr;
+
+        else
+            print = NULL;
+
+        print_ok = TRUE;
+    }
+
+    return print;
+}
+
+static void update_progress_nogui(gint id, gint count)
+{
+    FILE *print = get_terminal_output();
+
+    if (print != NULL && need_refresh_progress(id == count - 1)) {
+        gint percent = id * 100 / count;
+
+        fprintf(print, "  %d/%d: %d%%\r", id + 1, count, percent);
+        fflush(print);
+    }
+}
+
 static void update_progress(const gchar * filename, gint id, gint count)
 {
     const gchar *base;
-    static GTimeVal last = { 0, 0 };
     time_t current_time, elapsed_time, total_time, remaining_time;
     size_t len;
     gchar *ratio_text;
 
+    if (progress_window == NULL) {
+        update_progress_nogui(id, count);
+        return;
+    }
+
     if (filename == NULL) {
         /* Last update. */
         gchar *msg = _("Inserting files...");
 
-        last.tv_sec = last.tv_usec = 0;
+        need_refresh_progress(TRUE);
         gtk_label_set_text(file_label, msg);
 
     } else {
-        GTimeVal now;
-        g_get_current_time(&now);
-
-        if (last.tv_sec != 0 || last.tv_usec != 0) {
-            /* Not the first time. */
-            glong usec_diff;
-
-            g_get_current_time(&now);
-            usec_diff = (now.tv_sec - last.tv_sec) * G_USEC_PER_SEC +
-                now.tv_usec - last.tv_usec;
-
-            if (usec_diff < G_USEC_PER_SEC / 10)
-                /* Lower the redrawing load. */
-                return;
-        }
-
-        last = now;
+        if (need_refresh_progress(FALSE) == FALSE)
+            return;
 
         base = strrchr(filename, '/');
         if (base == NULL)
@@ -466,25 +509,26 @@ static gboolean align_next(FILE * file, gint value)
 }
 
 /* Write the MaybeThumb part of an entry. */
-static gboolean write_thumb(tree_item * item, FILE * file)
+static gboolean write_thumb(const gchar * filename, FILE * file)
 {
     gint height, rowstride, len, bps;
+    gchar *thumb_key;
     GdkPixbuf *thumb;
 
-    if (fill_thumbnail(item) == FALSE) {
+    thumb = get_thumbnail(filename, &thumb_key);
+    if (thumb == NULL) {
         WRITE(file, "0");
         return TRUE;
     }
 
-    thumb = item->thumb;
     height = gdk_pixbuf_get_height(thumb);
     rowstride = gdk_pixbuf_get_rowstride(thumb);
     len = pixels_size(thumb);
     bps = gdk_pixbuf_get_bits_per_sample(thumb);
 
     WRITE(file, "1");
-    WRITE_NB(file, strlen(item->thumb_key));    /* HashKeyLength */
-    WRITE(file, item->thumb_key);       /* HashKey */
+    WRITE_NB(file, strlen(thumb_key));  /* HashKeyLength */
+    WRITE(file, thumb_key);     /* HashKey */
     WRITE_NB(file, gdk_pixbuf_get_colorspace(thumb));   /* GdkColorspace */
     WRITE_NB(file, gdk_pixbuf_get_has_alpha(thumb));    /* AlphaChannel */
     WRITE_NB(file, bps);        /* BitsPerSample */
@@ -501,22 +545,22 @@ static gboolean write_thumb(tree_item * item, FILE * file)
     return fwrite(gdk_pixbuf_get_pixels(thumb), 1, len, file) == len;
 }
 
-static gboolean serialize_file(GNode * tree, serialization_data * work)
+static gboolean serialize_file(const gchar * filename,
+                               serialization_data * work)
 {
-    tree_item *item = tree->data;
     FILE *file = work->file;
     gchar *path;
 
-    update_progress(item->path, work->file_id, work->count);
+    update_progress(filename, work->file_id, work->count);
 
     WRITE(file, "<FILE>");
 
-    path = get_absolute_filename(item->path);
+    path = get_absolute_filename(filename);
     WRITE_NB(file, strlen(path));       /* PathLength */
     WRITE(file, path);          /* Path */
     g_free(path);
 
-    if (write_thumb(item, file) == FALSE)       /* MaybeThumb */
+    if (write_thumb(filename, file) == FALSE)   /* MaybeThumb */
         return FALSE;
 
     WRITE(file, "</FILE>");
@@ -525,106 +569,138 @@ static gboolean serialize_file(GNode * tree, serialization_data * work)
     return TRUE;
 }
 
-/* The tree is reversed, so we traverse it in the reverse direction. */
-static gboolean reverse_traverse_leafs(GNode * tree,
-                                       GNodeTraverseFunc func, gpointer data)
+static void file_error(const gchar * filename)
 {
-    GNode *child;
-
-    if (G_NODE_IS_LEAF(tree))
-        return func(tree, data);
-
-    child = g_node_last_child(tree);
-    while (child != NULL) {
-        if (canceled_using_tree())
-            return FALSE;
-
-        if (reverse_traverse_leafs(child, func, data) == FALSE)
-            return FALSE;
-
-        child = child->prev;
+    if (rt) {
+        gchar *message = g_strconcat(filename, ": ", g_strerror(errno), NULL);
+        DIALOG_MSG(message);
+        g_free(message);
     }
-
-    return TRUE;
 }
 
-#define FILE_ERROR(filename)                                    \
-    do {                                                        \
-        const gchar *message = g_strerror(errno);               \
-        message = g_strconcat((filename), ": ", message, NULL); \
-        DIALOG_MSG(message);                                    \
-    } while (0)
+gint save_collection(FILE * file, gboolean * cancel)
+{
+    GList *list;
+    serialization_data *work;
+    gboolean ok = FALSE;
 
-gboolean serialize_collection(void)
+    list = get_list_head();
+    is_saving = TRUE;
+    work = g_new(serialization_data, 1);
+    work->file = file;
+    work->file_id = 0;
+    work->count = get_list_length();
+    work->cwd = g_get_current_dir();
+
+    if (print_header(work) == FALSE)
+        goto error;
+
+    while (*cancel == FALSE && list != NULL) {
+        gboolean file_ok = serialize_file(list->data, work);
+        if (file_ok == FALSE)
+            goto error;
+
+        list = list->next;
+    }
+
+    if (*cancel || print_footer(work->file) == FALSE)
+        goto error;
+
+    ok = TRUE;
+    goto ok;
+
+  error:
+    ok = FALSE;
+
+  ok:
+    g_free(work->cwd);
+    g_free(work);
+
+    return ok ? 0 : 1;
+}
+
+gint serialize_collection_nogui(const gchar * filename)
+{
+    FILE *tty;
+    FILE *file;
+    gint res;
+    gboolean cancel = FALSE;
+
+    if (get_list_head() == NULL) {
+        g_printerr(_("No images to put in a collection\n"));
+        return 1;
+    }
+
+    if (filename == NULL) {
+        file = stdout;
+        if (isatty(fileno(file))) {
+            g_printerr(_("GLiv won't write a collection to a terminal\n"));
+            return 1;
+        }
+    } else {
+        /* We remove the file, since we may have mmapped it */
+        remove(filename);
+
+        file = fopen(filename, "w");
+        if (file == NULL) {
+            perror(filename);
+            return 1;
+        }
+    }
+
+    tty = get_terminal_output();
+    fprintf(tty, _("Saving collection: %s"), filename_to_utf8(filename));
+    putc('\n', tty);
+
+    res = save_collection(file, &cancel);
+
+    if (res)
+        perror(filename == NULL ? _("Standard output") : filename);
+
+    if (filename != NULL && fclose(file) < 0 && res == 0)
+        perror(filename);
+
+    return res;
+}
+
+gboolean serialize_collection_gui(void)
 {
     FILE *file;
-    GNode *tree;
     gchar *filename;
-    serialization_data *work;
-    gboolean res;
+    gint res;
+    gboolean cancel = FALSE;
 
-    tree = make_tree();
-    if (tree == NULL)
+    if (get_list_head() == NULL)
         return FALSE;
 
     filename = get_collection_destination();
-    if (filename == NULL) {
-        end_using_tree();
+    if (filename == NULL)
         return FALSE;
-    }
 
     /* We remove the file, since we may have mmapped it */
     remove(filename);
 
     file = fopen(filename, "w");
     if (file == NULL) {
-        FILE_ERROR(filename);
-        end_using_tree();
+        file_error(filename);
         return FALSE;
     }
 
-    create_progress_dialog(filename, TRUE, NULL);
+    create_progress_dialog(filename, &cancel);
 
-    work = g_new(serialization_data, 1);
-    work->file = file;
-    work->file_id = 0;
-    work->count = tree_count_files(tree);
-    work->cwd = g_get_current_dir();
+    res = save_collection(file, &cancel);
+    if (res) {
+        if (cancel == FALSE)
+            file_error(filename);
 
-    if (print_header(work) == FALSE)
-        goto error;
+        fclose(file);
+        remove(filename);
+    } else if (fclose(file) < 0)
+        file_error(filename);
 
-    res = reverse_traverse_leafs(tree,
-                                 (GNodeTraverseFunc) serialize_file, work);
-
-    if (res == FALSE || work->file_id != work->count)
-        goto error;
-
-    if (print_footer(work->file) == FALSE)
-        goto error;
-
-    if (fclose(work->file)) {
-        perror(filename);
-        work->file = NULL;
-        goto error;
-    }
-    work->file = NULL;
-
-    goto ok;
-
-  error:
-    if (canceled_using_tree() == FALSE || remove(filename) < 0)
-        FILE_ERROR(filename);
-
-  ok:
-    if (work->file != NULL)
-        fclose(work->file);
-
-    g_free(work->cwd);
-    g_free(work);
-    end_using_tree();
     destroy_progress_dialog();
     g_free(filename);
+
     return FALSE;
 }
 
@@ -883,7 +959,7 @@ static void destroy_source(struct coll_src *source, gboolean ok)
             perror("munmap");
 }
 
-gint load_dot_gliv_from_file(FILE * file, gboolean in_the_head)
+gint load_dot_gliv_from_file(FILE * file)
 {
     gint count, i = 0;
     tree_item **items = NULL;
@@ -891,7 +967,9 @@ gint load_dot_gliv_from_file(FILE * file, gboolean in_the_head)
     gboolean ok = TRUE, cancel = FALSE;
     gint nb_inserted = -1;
     struct coll_src *source;
+    gboolean saved_force_load;
 
+    is_saving = FALSE;
     source = build_source(file);
 
     CHECK_MAGIC(source, "GLiv <COLLECTION>");
@@ -901,7 +979,8 @@ gint load_dot_gliv_from_file(FILE * file, gboolean in_the_head)
     if (count < 0)
         goto error;
 
-    create_progress_dialog(NULL, FALSE, &cancel);
+    if (rt)
+        create_progress_dialog(NULL, &cancel);
 
     items = g_new(tree_item *, count);
     for (i = 0; i < count; i++) {
@@ -926,22 +1005,22 @@ gint load_dot_gliv_from_file(FILE * file, gboolean in_the_head)
     update_progress(NULL, count - 1, count);
     nb_inserted = 0;
 
-    if (in_the_head == FALSE)
-        filenames = g_new(gchar *, count);
+    filenames = g_new(gchar *, count);
 
     for (i = 0; i < count; i++) {
         if (items[i]->thumb_key != NULL)
             collection_add_thumbnail(items[i]->thumb_key, items[i]->thumb);
 
-        if (in_the_head)
-            nb_inserted += add_file_to_list(items[i]->path);
-        else
-            filenames[i] = items[i]->path;
+        filenames[i] = items[i]->path;
     }
 
-    if (in_the_head == FALSE)
-        nb_inserted = insert_after_current(filenames, count,
-                                           FALSE, FALSE, FALSE, FALSE);
+    /* We don't want to stat() every file in the collection */
+    saved_force_load = options->force;
+    options->force = TRUE;
+
+    nb_inserted = insert_after_current(filenames, count, TRUE, FALSE);
+
+    options->force = saved_force_load;
 
     goto ok;
 
@@ -961,35 +1040,43 @@ gint load_dot_gliv_from_file(FILE * file, gboolean in_the_head)
     }
     g_free(items);
     g_free(filenames);
-    destroy_progress_dialog();
+
+    if (rt)
+        destroy_progress_dialog();
     destroy_source(source, ok);
     return nb_inserted;
 }
 
-gint load_dot_gliv(const gchar * filename, gboolean in_the_head)
+gint load_dot_gliv(const gchar * filename)
 {
     FILE *file;
+    FILE *tty;
     gint nb_inserted;
     loader_t loader;
 
+    tty = get_terminal_output();
+    fprintf(tty, _("Loading collection: %s"), filename_to_utf8(filename));
+    putc('\n', tty);
+
     loader = get_loader(filename);
     if (loader == LOADER_DECOMP_DOT_GLIV)
-        nb_inserted = load_compressed_collection(filename, in_the_head);
+        nb_inserted = load_compressed_collection(filename);
 
     else {
         file = fopen(filename, "r");
         if (file == NULL) {
-            FILE_ERROR(filename);
+            file_error(filename);
             return 0;
         }
 
-        nb_inserted = load_dot_gliv_from_file(file, in_the_head);
+        nb_inserted = load_dot_gliv_from_file(file);
         fclose(file);
     }
 
     if (nb_inserted < 0) {
-        DIALOG_MSG(_("%s is not a GLiv collection"),
-                   filename_to_utf8(filename));
+        if (rt)
+            DIALOG_MSG(_("%s is not a GLiv collection"),
+                       filename_to_utf8(filename));
         nb_inserted = 0;
     }
 
@@ -1001,11 +1088,11 @@ gboolean deserialize_collection(void)
     gchar *filename;
     gint nb_inserted;
 
-    filename = get_collection_source();
+    filename = get_filename(_("Choose a collection to load"), FALSE);
     if (filename == NULL)
         return FALSE;
 
-    nb_inserted = load_dot_gliv(filename, FALSE);
+    nb_inserted = load_dot_gliv(filename);
     g_free(filename);
     if (nb_inserted <= 0)
         return FALSE;
