@@ -22,47 +22,13 @@
  * A simplified ftw(3), specific to GLiv *
  *****************************************/
 
-#include <sys/types.h>          /* dev_t, ino_t */
 #include <sys/stat.h>           /* struct stat, stat() */
 #include <dirent.h>             /* struct dirent, *dir() */
 #include <stdio.h>              /* perror() */
 
 #include "gliv.h"
 #include "foreach_file.h"
-
-/* Used to detect insertions of already present elements. */
-static gboolean destroyed_node = FALSE;
-
-/*
- * Directories informations saved into
- * the AVL to avoid infinite loops.
- */
-typedef struct {
-    dev_t dev;
-    ino_t ino;
-} dir_info;
-
-/* Called when we insert an element that was already there. */
-static void value_destroy_notify(gpointer data)
-{
-    g_free(data);
-    destroyed_node = TRUE;
-}
-
-/* To build and search the AVL. */
-static gint cmp_func(const dir_info * dir1, const dir_info * dir2)
-{
-    if (dir1->ino < dir2->ino)
-        return -1;
-
-    if (dir1->ino > dir2->ino)
-        return 1;
-
-    if (dir1->dev < dir2->dev)
-        return -1;
-
-    return dir1->dev > dir2->dev;
-}
+#include "pathset.h"
 
 /* Push directories and handle files. */
 static GSList *process_dir(GSList * stack, const gchar * dirname,
@@ -94,6 +60,7 @@ static GSList *process_dir(GSList * stack, const gchar * dirname,
 #ifdef _DIRENT_HAVE_D_TYPE
         switch (dir_ent->d_type) {
         case DT_UNKNOWN:
+        case DT_LNK:
             break;
 
         case DT_DIR:
@@ -129,27 +96,6 @@ static GSList *process_dir(GSList * stack, const gchar * dirname,
     return stack;
 }
 
-/* Return TRUE if the dir has been correctly inserted. */
-static gboolean tree_insert_dir(GTree * tree, struct stat *st)
-{
-    dir_info *dir;
-
-    dir = g_new(dir_info, 1);
-
-    dir->dev = st->st_dev;
-    dir->ino = st->st_ino;
-
-    g_tree_insert(tree, dir, NULL);
-
-    if (destroyed_node) {
-        /* It was already there. */
-        destroyed_node = FALSE;
-        return FALSE;
-    }
-
-    return TRUE;
-}
-
 /*
  * We know that func is add_file_to_list and
  * we return the number of inserted files.
@@ -157,8 +103,8 @@ static gboolean tree_insert_dir(GTree * tree, struct stat *st)
 gint foreach_file(const gchar * path, foreach_file_func func)
 {
     GSList *stack = NULL;
-    GTree *tree;
     gchar *current;
+    struct pathset *set;
     struct stat st;
     gint res = 0;
 
@@ -176,22 +122,20 @@ gint foreach_file(const gchar * path, foreach_file_func func)
 
     stack = g_slist_prepend(stack, g_strdup(path));     /* push */
 
-    tree = g_tree_new_full((GCompareDataFunc) cmp_func, NULL,
-                           g_free, value_destroy_notify);
+    set = pathset_new();
 
     while (stack) {
         current = stack->data;
         stack = g_slist_remove_link(stack, stack);      /* pop */
 
-        if (!stat(current, &st) && tree_insert_dir(tree, &st))
+        if (!stat(current, &st) && pathset_add(set, current))
             /* Not already scanned. */
             stack = process_dir(stack, current, func, &res);
 
         g_free(current);
     }
 
-    g_tree_destroy(tree);
-    destroyed_node = FALSE;
+    pathset_free(set);
 
     return res;
 }
