@@ -22,6 +22,7 @@
  * Creation and callbacks of the OpenGL widget *
  ***********************************************/
 
+#include "opengl.h"
 #include <GL/glu.h>
 #include <gtk/gtkgl.h>
 
@@ -37,7 +38,6 @@
 #include "next_image.h"
 #include "images_menus.h"
 #include "cursors.h"
-#include "opengl.h"
 
 extern GtkWidget *gl_widget;
 extern rt_struct *rt;
@@ -162,7 +162,7 @@ static gboolean drag_data_received(GtkWidget * unused1,
     }
 
     ok = TRUE;
-    nb_inserted = insert_after_current(uris, i, FALSE, FALSE, TRUE, TRUE);
+    nb_inserted = insert_after_current(uris, i, FALSE, TRUE);
     if (nb_inserted > 0)
         open_next_image(nb_inserted == 1);
 
@@ -302,12 +302,12 @@ static gboolean first_expose(void)
     first_image = load_first_image();
     if (first_image != NULL) {
         if (options->fullscreen == FALSE && options->resize_win == FALSE)
-            resize_window(first_image->width, first_image->height, TRUE);
+            goto_window(first_image, TRUE);
 
         g_object_unref(first_image);
 
     } else if (options->fullscreen == FALSE)
-        resize_window(-1, -1, TRUE);
+        goto_window(NULL, TRUE);
 
     load_second_image();
 
@@ -401,7 +401,7 @@ gint report_opengl_errors(const gchar * function_name,
     }
 
     if (errors != NULL) {
-        ERROR_MSG("%s\n", errors);
+        g_printerr("%s\n", errors);
         g_free(errors);
     }
 
