@@ -41,7 +41,7 @@ static gchar *get_help_text(void)
 {
     gchar **help_lines, **ptr;
     gchar *res, *res_ptr;
-    int len = 0;
+    gint len = 0;
 
     help_lines = get_help_lines();
 
@@ -106,7 +106,7 @@ static void show_help(void)
 
     g_signal_connect(help_win, "delete-event", G_CALLBACK(toggle_help), NULL);
 
-    show_dialog(GTK_WIDGET(help_win), _("GLiv help"));
+    show_message(GTK_WIDGET(help_win), _("GLiv help"));
 }
 
 gboolean toggle_help(void)
