@@ -218,7 +218,7 @@ static DirtyGtkMenu *add_sub_menu(DirtyGtkMenu * parent, tree_item * item)
     menu_item = add_menu_item(parent, item, MENU_SUBMENU);
 
     menu = dirty_gtk_menu_new();
-    gtk_menu_item_set_submenu(menu_item, GTK_WIDGET(menu->gtk_menu_shell));
+    gtk_menu_item_set_submenu(menu_item, GTK_WIDGET(menu->gtk_menu));
 
     return menu;
 }
@@ -338,7 +338,7 @@ static DirtyGtkMenu *begin_rebuild(GtkMenuItem * root, GCallback func)
     g_signal_connect_swapped(rebuilder, "activate", func, NULL);
     dirty_gtk_menu_append(menu, rebuilder);
 
-    gtk_menu_item_set_submenu(root, GTK_WIDGET(menu->gtk_menu_shell));
+    gtk_menu_item_set_submenu(root, GTK_WIDGET(menu->gtk_menu));
 
     return menu;
 }
@@ -377,24 +377,29 @@ static GNode *get_tree(void)
         set_menu_indicator(name, 0);                                         \
     } while (0)
 
-#define REBUILD_END()                                                        \
-    do {                                                                     \
-        if (root != NULL)                                                    \
-            gtk_menu_item_deselect(root);                                    \
-                                                                             \
-        set_menu_indicator(NULL, 0);                                         \
-        gtk_widget_set_sensitive(GTK_WIDGET(cancel_menu_item), FALSE);       \
-        if (canceled_using_tree())                                           \
-            reset_timestamp(&timestamp);                                     \
-        else                                                                 \
-            touch(&timestamp);                                               \
-        end_using_tree();                                                    \
-        reset_mnemonics();                                                   \
-        gtk_widget_show_all(GTK_WIDGET(menu_item));                          \
-        dirty_gtk_menu_release(menu);                                        \
-        tree_count = 0;                                                      \
-        return timestamp != 0;                                               \
-    } while (0)
+static gboolean rebuild_end(GtkMenuItem * root, timestamp_t * ts,
+                            GtkMenuItem * menu_item, DirtyGtkMenu * menu)
+{
+    if (root != NULL)
+        gtk_menu_item_deselect(root);
+
+    dirty_gtk_menu_append(menu, gtk_tearoff_menu_item_new());
+
+    set_menu_indicator(NULL, 0);
+    gtk_widget_set_sensitive(GTK_WIDGET(cancel_menu_item), FALSE);
+
+    if (canceled_using_tree())
+        reset_timestamp(ts);
+    else
+        touch(ts);
+
+    end_using_tree();
+    reset_mnemonics();
+    gtk_widget_show_all(GTK_WIDGET(menu_item));
+    dirty_gtk_menu_release(menu);
+    tree_count = 0;
+    return *ts != 0;
+}
 
 gboolean rebuild_directories(GtkMenuItem * root)
 {
@@ -428,7 +433,7 @@ gboolean rebuild_directories(GtkMenuItem * root)
     } else
         make_menu_from_tree_rec(tree, menu);
 
-    REBUILD_END();
+    return rebuild_end(root, &timestamp, menu_item, menu);
 }
 
 gboolean rebuild_images(GtkMenuItem * root)
@@ -445,7 +450,7 @@ gboolean rebuild_images(GtkMenuItem * root)
     g_node_traverse(tree, G_PRE_ORDER, G_TRAVERSE_LEAFS, -1,
                     (GNodeTraverseFunc) add_file_item, menu);
 
-    REBUILD_END();
+    return rebuild_end(root, &timestamp, menu_item, menu);
 }
 
 /* Rebuild both menus. */
