@@ -11,14 +11,15 @@ gint add_file_to_list(const gchar * filename);
 void remove_from_list(GList * node);
 gboolean reorder_files(gboolean shuffle);
 gchar **get_sorted_files_array(void);
-void init_list(gchar ** names, gint num, gboolean recursive,
-               gboolean shuffle, gboolean sort);
-void read_null_filenames(gboolean recursive, gboolean shuffle, gboolean sort);
-void place_next(const gchar * filename);
-gint insert_after_current(gchar ** names, gint nb, gboolean recursive,
-                          gboolean shuffle, gboolean sort, gboolean do_stat);
+gint init_from_null_filenames(gboolean sort, gboolean shuffle,
+                              gboolean add_all);
+gint init_list(gchar ** names, gint nb, gboolean sort, gboolean shuffle,
+               gboolean add_all);
+gint insert_after_current(gchar ** names, gint nb, gboolean just_file,
+                          gboolean add_all);
 gboolean confirm_remove_current(void);
 void delete_selected_image(void);
 timestamp_t get_list_timestamp(void);
+const gchar *get_nth_filename(gint n);
 
 #endif
