@@ -5,6 +5,8 @@
 #include "gliv-image.h"
 #include "formats.h"
 
+void start_quiet(void);
+void stop_quiet(void);
 gboolean is_loading(void);
 void install_segv_handler(void);
 void fill_ident(GlivImage * im);
