@@ -6,12 +6,7 @@
 #include "cursors.h"
 
 #include "gettext.h"
-#define _(str) (gettext((str)))
-
-/* Just to mark constant translatable strings, like in rcfile.c. */
-#define N_(str) (str)
-
-#define ERROR_MSG g_printerr
+#include <glib/gi18n.h>
 
 #if HAVE_ISO_C_VARARGS
 
@@ -19,7 +14,7 @@
     do {                                                           \
         GtkMessageDialog *__dialog__;                              \
                                                                    \
-        ERROR_MSG(__VA_ARGS__);                                    \
+        g_printerr(__VA_ARGS__);                                   \
         g_printerr("\n");                                          \
                                                                    \
         __dialog__ = GTK_MESSAGE_DIALOG(                           \
@@ -41,7 +36,7 @@
     } while (0)
 
 #else
-#define DIALOG_MSG ERROR_MSG
+#define DIALOG_MSG g_printerr
 #warning "No variadic macros"
 #endif
 
