@@ -10,11 +10,12 @@ gboolean load_direction(gint dir);
 void open_next_image(gboolean keep_next);
 GlivImage *load_first_image(void);
 void load_second_image(void);
-void open_file(const gchar * name, gboolean add_dir, gboolean shuffle_flag);
-void reload_all_images(void);
+void reload_images(void);
 gboolean menu_load(const gchar * filename);
 void unload(GList * node);
 void unload_images(void);
+gpointer stat_loaded_files(void);
+void reload_changed_files(gpointer * stat_data);
 void set_slide_show_menu(GtkMenuItem * item);
 void start_slide_show(void);
 gboolean slide_show_started(void);
