@@ -11,7 +11,6 @@
 #define OPTION_DELAY_CURSOR              (&new_options->delay)
 #define OPTION_DITHERING                 (&new_options->dither)
 #define OPTION_SCALE_DOWN                (&new_options->scaledown)
-#define OPTION_FORCE_LOAD                (&new_options->force)
 #define OPTION_ALPHA_CHECKS              (&new_options->alpha_checks)
 #define OPTION_ONE_IMAGE                 (&new_options->one_image)
 #define OPTION_MIPMAPS                   (&new_options->mipmap)
@@ -63,6 +62,7 @@ typedef struct {
     gboolean confirm_quit;
     gboolean save_quit;
     gboolean transitions;
+    gboolean recursive;
     gint thumb_width;
     gint thumb_height;
     gint delay;
