@@ -32,10 +32,8 @@
 
 #include "gliv.h"
 #include "ipc.h"
-#include "cmdline.h"
 #include "files_list.h"
 #include "next_image.h"
-#include "main.h"
 
 /* The current server. */
 static guint watch_id;
@@ -80,28 +78,10 @@ static gchar *get_socket_name(void)
 
 /*** State encoding by the client for the server. ***/
 
-/* Send or receive a gboolean. */
-#define PARAM(func, param)                                               \
-    do {                                                                 \
-        gsize bytes = 0;                                                 \
-        GError *err = NULL;                                              \
-                                                                         \
-        func(channel, (gpointer) param, sizeof(gboolean), &bytes, &err); \
-        if (bytes != sizeof(gboolean) && err != NULL) {                  \
-            g_printerr("%s\n", err->message);                            \
-            g_error_free(err);                                           \
-            return FALSE;                                                \
-        }                                                                \
-    } while (0)
-
 /*
- * Before sending the '\0' filenames we send three command line flags:
- * recursive, shuffle and sort.
- * We also send the working directory in order to deal with relative
- * filenames.
+ * We send the working directory in order to deal with relative filenames.
  */
-static gboolean write_params(GIOChannel * channel, gboolean recursive,
-                             gboolean shuffle, gboolean sort)
+static gboolean write_param(GIOChannel * channel)
 {
     gchar *current_dir;
     gint length;
@@ -109,10 +89,6 @@ static gboolean write_params(GIOChannel * channel, gboolean recursive,
     gsize bytes = 0;
     GError *err = NULL;
 
-    PARAM(g_io_channel_write_chars, &recursive);
-    PARAM(g_io_channel_write_chars, &shuffle);
-    PARAM(g_io_channel_write_chars, &sort);
-
     current_dir = g_get_current_dir();
     length = strlen(current_dir) + 1;
 
@@ -134,19 +110,13 @@ static gboolean write_params(GIOChannel * channel, gboolean recursive,
  * We read what the function above sent. If the server and the client are
  * running in the same directory we put NULL in *client_dir.
  */
-static gboolean read_params(GIOChannel * channel, gchar ** client_dir,
-                            gboolean * recursive,
-                            gboolean * shuffle, gboolean * sort)
+static gboolean read_param(GIOChannel * channel, gchar ** client_dir)
 {
     size_t size = 0;
     gchar *server_dir;
     struct stat client_stat, server_stat;
     GError *err = NULL;
 
-    PARAM(g_io_channel_read_chars, recursive);
-    PARAM(g_io_channel_read_chars, shuffle);
-    PARAM(g_io_channel_read_chars, sort);
-
     *client_dir = NULL;
     g_io_channel_read_line(channel, client_dir, &size, NULL, &err);
     if (err != NULL) {
@@ -248,15 +218,13 @@ static gchar **rebuild_args_array(gchar * data, gint length, gint * count,
 }
 
 /* Add the filenames and open the next image. */
-static void process_data(gchar * data, gint length, gchar * client_dir,
-                         gboolean recursive, gboolean shuffle, gboolean sort)
+static void process_data(gchar * data, gint length, gchar * client_dir)
 {
     gint i, count, nb_inserted;
     gchar **tab;
 
     tab = rebuild_args_array(data, length, &count, client_dir);
-    nb_inserted = insert_after_current(tab, count,
-                                       recursive, shuffle, sort, TRUE);
+    nb_inserted = insert_after_current(tab, count, TRUE, FALSE);
 
     if (nb_inserted > 0)
         open_next_image(nb_inserted == 1);
@@ -272,11 +240,10 @@ static gboolean handle_connect(GIOChannel * source, GIOCondition unused1,
                                gpointer unused2)
 {
     gchar *data;
-    guint length = 0;
+    gsize length = 0;
     gint fd;
     GError *err = NULL;
     GIOChannel *channel;
-    gboolean recursive, shuffle, sort;
     gchar *client_dir;
 
     fd = accept(g_io_channel_unix_get_fd(source), NULL, &length);
@@ -289,8 +256,8 @@ static gboolean handle_connect(GIOChannel * source, GIOCondition unused1,
     if (channel == NULL)
         return TRUE;
 
-    /* Fill the flags. */
-    if (!read_params(channel, &client_dir, &recursive, &shuffle, &sort)) {
+    /* Fill the param. */
+    if (!read_param(channel, &client_dir)) {
         g_io_channel_unref(channel);
         return TRUE;
     }
@@ -304,7 +271,7 @@ static gboolean handle_connect(GIOChannel * source, GIOCondition unused1,
         return TRUE;
     }
 
-    process_data(data, length, client_dir, recursive, shuffle, sort);
+    process_data(data, length, client_dir);
     g_io_channel_unref(channel);
     return TRUE;
 }
@@ -367,72 +334,35 @@ static gint create_client_socket(void)
     return sock;
 }
 
-/* Used when we read the filenames from stdin. */
-static void send_filenames(GIOChannel * channel)
-{
-    gchar buffer[4096];
-    gint size;
-
-    while ((size = fread(buffer, 1, sizeof(buffer), stdin)) > 0)
-        g_io_channel_write_chars(channel, buffer, size, NULL, NULL);
-}
-
-static void fill_flags(struct gengetopt_args_info *ggo,
-                       gboolean * recursive, gboolean * shuffle,
-                       gboolean * sort, gboolean * null)
-{
-    if (ggo->recursive_given)
-        *recursive = get_on_off(ggo->recursive_arg);
-
-    if (ggo->shuffle_given)
-        *shuffle = get_on_off(ggo->shuffle_arg);
-
-    if (ggo->sort_given)
-        *sort = get_on_off(ggo->sort_arg);
-
-    if (ggo->null_given)
-        *null = get_on_off(ggo->null_arg);
-}
-
-gboolean connect_server(gint argc, gchar ** argv)
+gboolean connect_server(void)
 {
     gboolean ret = TRUE;
-    gint fd, i;
-    struct gengetopt_args_info ggo;
+    gint fd;
     GIOChannel *channel;
-    gboolean recursive_flag = FALSE, shuffle_flag = FALSE,
-        sort_flag = FALSE, null_flag = FALSE;
+    GList *list;
 
     fd = create_client_socket();
     if (fd < 0)
         return FALSE;
 
-    /* To extract the four flags. */
-    cmdline_parser(argc, argv, &ggo);
-    fill_flags(&ggo, &recursive_flag, &shuffle_flag, &sort_flag, &null_flag);
-
     channel = make_channel(fd);
     if (channel == NULL)
         goto out;
 
-    /* Send the three flags and our current directory. */
-    if (!write_params(channel, recursive_flag, shuffle_flag, sort_flag)) {
+    /* Send our current directory. */
+    if (!write_param(channel)) {
         g_io_channel_unref(channel);
         ret = FALSE;
         goto out;
     }
 
-    /* gliv -0 */
-    if (null_flag)
-        send_filenames(channel);
-
     /* Send all the filenames with '\0' between them. */
-    for (i = 0; i < ggo.inputs_num; i++) {
+    for (list = get_list_head(); list != NULL; list = list->next) {
         gsize size;
-        gint length = strlen(ggo.inputs[i]) + 1;
+        gint length = strlen(list->data) + 1;
         GError *err = NULL;
 
-        g_io_channel_write_chars(channel, ggo.inputs[i], length, &size, &err);
+        g_io_channel_write_chars(channel, list->data, length, &size, &err);
         if (err != NULL) {
             g_printerr("%s\n", err->message);
             g_error_free(err);
@@ -444,6 +374,5 @@ gboolean connect_server(gint argc, gchar ** argv)
     g_io_channel_unref(channel);
 
   out:
-    cmdline_parser_free(&ggo);
     return ret;
 }
