@@ -31,7 +31,6 @@
 #include "options.h"
 #include "gliv-image.h"
 #include "messages.h"
-#include "gl_widget.h"
 #include "str_utils.h"
 #include "formats.h"
 #include "params.h"
@@ -48,6 +47,22 @@ extern options_struct *options;
 /* Used to know at any time if we are loading an image. */
 static const gchar *loading_filename = NULL;
 
+/*
+ * Used to be quiet when we try to load files
+ * not explicitly requested by the user.
+ */
+static gint quiet = 0;
+
+void start_quiet(void)
+{
+    quiet++;
+}
+
+void stop_quiet(void)
+{
+    quiet--;
+}
+
 gboolean is_loading(void)
 {
     return loading_filename != NULL;
@@ -61,10 +76,10 @@ static void segv_handler(gint sig)
     sig_str = g_strsignal(sig);
 
     if (loading_filename == NULL)
-        ERROR_MSG(_("%s not while loading an image\n"), sig_str);
+        g_printerr(_("%s not while loading an image\n"), sig_str);
     else
-        ERROR_MSG(_("%s while loading %s\n"),
-                  sig_str, filename_to_utf8(loading_filename));
+        g_printerr(_("%s while loading %s\n"),
+                   sig_str, filename_to_utf8(loading_filename));
 
     raise(sig);
 }
@@ -112,6 +127,8 @@ const gchar *get_extension(const gchar * filename)
     return ext;
 }
 
+#define LOADING_ERROR if (!quiet) g_printerr
+
 static loader_t get_decompressor(const gchar * filename, gint ext_len)
 {
     const struct format *res;
@@ -134,15 +151,15 @@ static loader_t get_decompressor(const gchar * filename, gint ext_len)
     g_free(image_ext);
 
     if (res == NULL) {
-        ERROR_MSG(_("%s: unknown decompressed extension\n"),
-                  filename_to_utf8(filename));
+        LOADING_ERROR(_("%s: unknown decompressed extension\n"),
+                      filename_to_utf8(filename));
         return LOADER_NONE;
     }
 
     switch (res->loader) {
     case LOADER_DECOMP:
-        ERROR_MSG(_("%s: image cannot be compressed twice\n"),
-                  filename_to_utf8(filename));
+        LOADING_ERROR(_("%s: image cannot be compressed twice\n"),
+                      filename_to_utf8(filename));
         return LOADER_NONE;
 
     case LOADER_PIXBUF:
@@ -164,8 +181,8 @@ loader_t get_loader(const gchar * filename)
 
     ext = get_extension(filename);
     if (ext == NULL) {
-        ERROR_MSG(_("%s: unknown extension (none)\n"),
-                  filename_to_utf8(filename));
+        LOADING_ERROR(_("%s: unknown extension (none)\n"),
+                      filename_to_utf8(filename));
         return LOADER_NONE;
     }
 
@@ -173,7 +190,7 @@ loader_t get_loader(const gchar * filename)
     res = ext_to_loader(ext, ext_len);
 
     if (res == NULL) {
-        ERROR_MSG(_("%s: unknown extension\n"), filename_to_utf8(filename));
+        LOADING_ERROR(_("%s: unknown extension\n"), filename_to_utf8(filename));
         return LOADER_NONE;
     }
 
