@@ -1,4 +1,3 @@
-
 /*
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -37,6 +36,7 @@
 #include "messages.h"
 #include "windows.h"
 #include "ipc.h"
+#include "collection.h"
 
 rt_struct *rt;
 options_struct *options;
@@ -44,8 +44,10 @@ GlivImage *current_image = NULL;
 GtkWidget *gl_widget;
 GtkMenuBar *menu_bar;
 
+static gboolean option_add_all = FALSE;
+
 /* May exit :) */
-gboolean get_on_off(const gchar * str)
+static gboolean get_on_off(const gchar * str)
 {
     if (str == NULL)
         return TRUE;
@@ -59,7 +61,7 @@ gboolean get_on_off(const gchar * str)
     if (g_str_equal(str, "off"))
         return FALSE;
 
-    ERROR_MSG(_("Command line flags should be on or off, not %s\n"), str);
+    g_printerr(_("Command line flags should be on or off, not %s\n"), str);
     quit(1);
 }
 
@@ -81,37 +83,17 @@ static void fill_options(struct gengetopt_args_info *ggo)
 {
     ggo_flag flags[] = {
 /* *INDENT-OFF* */
-{ ggo->full_screen_given,  &options->fullscreen,   ggo->full_screen_arg,  0 },
-{ ggo->scale_down_given,   &options->scaledown,    ggo->scale_down_arg,   0 },
-{ ggo->maximize_given,     &options->maximize,     ggo->maximize_arg,     0 },
-{ ggo->zoom_pointer_given, &options->zoom_pointer, ggo->zoom_pointer_arg, 0 },
-{ ggo->dither_given,       &options->dither,       ggo->dither_arg,       0 },
+{ ggo->recursive_given,    &options->recursive,    ggo->recursive_arg,    0 },
 { ggo->force_load_given,   &options->force,        ggo->force_load_arg,   0 },
-{ ggo->mipmap_given,       &options->mipmap,       ggo->mipmap_arg,       0 },
 { ggo->build_menus_given,  &options->build_menus,  ggo->build_menus_arg,  1 },
-{ ggo->mnemonics_given,    &options->mnemonics,    ggo->mnemonics_arg,    1 },
-{ ggo->menu_given,         &options->menu_bar,     ggo->menu_arg,         1 },
-{ ggo->info_given,         &options->status_bar,   ggo->info_arg,         1 },
-{ ggo->scrollbars_given,   &options->scrollbars,   ggo->scrollbars_arg,   1 },
-{ ggo->alpha_checks_given, &options->alpha_checks, ggo->alpha_checks_arg, 1 },
-{ ggo->loop_given,         &options->loop,         ggo->loop_arg,         0 },
-{ ggo->one_image_given,    &options->one_image,    ggo->one_image_arg,    0 },
-{ ggo->thumbnails_given,   &options->thumbnails,   ggo->thumbnails_arg,   1 },
 { ggo->slide_show_given,   &options->start_show,   ggo->slide_show_arg,   0 },
-{ ggo->resize_win_given,   &options->resize_win,   ggo->resize_win_arg,   1 },
-{ ggo->make_fit_given,     &options->maximize,     ggo->make_fit_arg,     0 },
-{ ggo->make_fit_given,     &options->scaledown,    ggo->make_fit_arg,     0 },
-{ ggo->no_center_given,    &options->no_center,    ggo->no_center_arg,    0 },
+{ ggo->add_all_given,      &option_add_all,        ggo->add_all_arg,      0 },
 { FALSE,                   NULL,                   NULL,                    }
 /* *INDENT-ON* */
     };
 
     ggo_int ints[] = {
 /* *INDENT-OFF* */
-{ ggo->delay_given,        &options->delay,        ggo->delay_arg,     0 },
-{ ggo->history_given,      &options->history_size, ggo->history_arg,  -1 },
-{ ggo->duration_given,     &options->duration,     ggo->duration_arg,  0 },
-{ ggo->fps_given,          &options->fps,          ggo->fps_arg,      -1 },
 { FALSE,                   NULL,                   FALSE,              0 }
 /* *INDENT-ON* */
     };
@@ -133,84 +115,112 @@ static void fill_options(struct gengetopt_args_info *ggo)
 
 #define FLAG_ON(flag) (flag##_given && get_on_off(flag##_arg))
 
-static void init_args(gint argc, gchar ** argv)
+/*
+ * We temporarily close stderr because unknown and gtk arguments are handled
+ * afterwards.
+ */
+static struct gengetopt_args_info *parse_cmdline(gint argc, gchar ** argv,
+                                                 gboolean silent)
+{
+    struct gengetopt_args_info *ggo = g_new(struct gengetopt_args_info, 1);
+    gint fd = -1;
+
+    if (silent) {
+        /* Silence stderr. */
+        fd = dup(STDERR_FILENO);
+        close(STDERR_FILENO);
+    }
+
+    if (cmdline_parser(argc, argv, ggo) != 0) {
+        g_free(ggo);
+        ggo = NULL;
+    }
+
+    if (silent) {
+        /* Restore stderr. */
+        dup2(fd, STDERR_FILENO);
+        close(fd);
+    }
+
+    return ggo;
+}
+
+typedef enum {
+    INIT_BAD_CMDLINE,
+    INIT_OK,
+    INIT_NO_IMAGES
+} init_res;
+
+static init_res init_args(gint argc, gchar ** argv, gboolean gtk_initialized)
 {
-    struct gengetopt_args_info ggo;
     options_struct *rc_file;
+    gint nb_inserted = 0;
+    struct gengetopt_args_info *ggo =
+        parse_cmdline(argc, argv, !gtk_initialized);
+    init_res res = INIT_BAD_CMDLINE;
+
+    if (ggo == NULL)
+        return res;
+
+    if (!ggo->collection_given && !gtk_initialized)
+        /*
+         * We want a "graphical" gliv, so init it after gtk
+         * to be able to show the progress dialog when loading collections.
+         */
+        return res;
+
+
+    res = INIT_OK;
 
     /* Command line (some flags only). */
 
-    if (cmdline_parser(argc, argv, &ggo) != 0)
-        quit(1);
-
-    if (FLAG_ON(ggo.sort) && FLAG_ON(ggo.shuffle)) {
+    if (FLAG_ON(ggo->sort) && FLAG_ON(ggo->shuffle)) {
         g_printerr(_("Cannot sort and shuffle at the same time\n"));
         quit(1);
     }
 
-    if (FLAG_ON(ggo.client) && connect_server(argc, argv))
-        /* Successfully reused a GLiv window. */
-        quit(0);
-
     /* Configuration file. */
-    rc_file = load_rc(!FLAG_ON(ggo.glivrc));
+    rc_file = load_rc(!ggo->glivrc_given, ggo->glivrc_arg);
     options = g_memdup(rc_file, sizeof(options_struct));
 
     /* Command line (remaining options). */
-    fill_options(&ggo);
+    fill_options(ggo);
 
-    if (FLAG_ON(ggo.glivrc))
-        options->save_quit = FALSE;
+    if (gtk_initialized) {
+        /* We use the (rt == NULL) check to see if gtk is initialized or not */
+        rt = g_new(rt_struct, 1);
 
-    rt = g_new(rt_struct, 1);
+        rt->cursor_hidden = FALSE;
+        rt->help = FALSE;
+        rt->alpha_checks_changed = TRUE;
+    }
 
-    rt->cursor_hidden = FALSE;
-    rt->help = FALSE;
-    rt->alpha_checks_changed = TRUE;
-    rt->scr_width = gdk_screen_width();
-    rt->scr_height = gdk_screen_height();
+    if (FLAG_ON(ggo->null))
+        nb_inserted =
+            init_from_null_filenames(FLAG_ON(ggo->sort), FLAG_ON(ggo->shuffle),
+                                     option_add_all);
 
-    if (FLAG_ON(ggo.null)) {
-        read_null_filenames(FLAG_ON(ggo.recursive), FLAG_ON(ggo.shuffle),
-                            FLAG_ON(ggo.sort));
-
-        insert_after_current(ggo.inputs, ggo.inputs_num,
-                             FLAG_ON(ggo.recursive),
-                             FLAG_ON(ggo.shuffle), FLAG_ON(ggo.sort), TRUE);
-
-    } else if (ggo.inputs_num > 0)
+    if (ggo->inputs_num > 0) {
         /* There are filenames on the command line. */
-        init_list(ggo.inputs, ggo.inputs_num,
-                  FLAG_ON(ggo.recursive),
-                  FLAG_ON(ggo.shuffle), FLAG_ON(ggo.sort));
+        nb_inserted +=
+            init_list(ggo->inputs, ggo->inputs_num, FLAG_ON(ggo->sort),
+                      FLAG_ON(ggo->shuffle), option_add_all);
 
-    cmdline_parser_free(&ggo);
-}
+        if (nb_inserted == 0 && gtk_initialized)
+            res = INIT_NO_IMAGES;
+    }
 
-/*
- * We call cmdline_parser() a first time to quickly exit if --help or --version
- * was given.  We do the real work after the next call in init_args(), after
- * gtk_init() removed the gtk arguments.
- *
- * We temporarily close stderr because unknown and gtk arguments are handled
- * afterwards.
- */
-static void check_quick_exit(gint argc, gchar ** argv)
-{
-    struct gengetopt_args_info ggo;
-    gint fd;
+    if (ggo->collection_given)
+        quit(serialize_collection_nogui(ggo->collection_arg));
 
-    /* Silence stderr. */
-    fd = dup(STDERR_FILENO);
-    close(STDERR_FILENO);
+    if (FLAG_ON(ggo->client) && connect_server())
+        /* Successfully reused a GLiv window. */
+        quit(0);
 
-    cmdline_parser(argc, argv, &ggo);
+    cmdline_parser_free(ggo);
+    g_free(ggo);
 
-    /* Restore stderr. */
-    dup2(fd, STDERR_FILENO);
-    close(fd);
-
-    cmdline_parser_free(&ggo);
+    return res;
 }
 
 G_GNUC_NORETURN void quit(gint code)
@@ -231,7 +241,7 @@ gboolean gui_quit(void)
 
     msg = _("Do you really want to quit GLiv?");
 
-    dialog = GTK_MESSAGE_DIALOG(gtk_message_dialog_new(get_current_window(),
+    dialog = GTK_MESSAGE_DIALOG(gtk_message_dialog_new(NULL,
                                                        GTK_DIALOG_MODAL,
                                                        GTK_MESSAGE_QUESTION,
                                                        GTK_BUTTONS_YES_NO,
@@ -250,7 +260,7 @@ gboolean gui_quit(void)
 
 gint main(gint argc, gchar ** argv)
 {
-    check_quick_exit(argc, argv);
+    init_res res;
 
     /* i18n */
 #ifdef ENABLE_NLS
@@ -263,10 +273,30 @@ gint main(gint argc, gchar ** argv)
 #endif
 
     g_thread_init(NULL);
+    g_type_init();
+
+    res = init_args(argc, argv, FALSE);
+
     gtk_init(&argc, &argv);
     gtk_gl_init(&argc, &argv);
 
-    init_args(argc, argv);
+    if (res == INIT_BAD_CMDLINE)
+        /* Now that GTK may have removed some arguments */
+        res = init_args(argc, argv, TRUE);
+
+    switch (res) {
+    case INIT_NO_IMAGES:
+        DIALOG_MSG(_("No image found"));
+        break;
+
+    case INIT_BAD_CMDLINE:
+        quit(1);
+        break;
+
+    case INIT_OK:
+        /* OK */
+        break;
+    }
 
     create_windows();
     start_server();
