@@ -47,6 +47,7 @@
 #include "collection.h"
 #include "ipc.h"
 #include "actions.h"
+#include "tree.h"
 
 extern options_struct *options;
 extern GtkMenuBar *menu_bar;
@@ -355,7 +356,7 @@ GtkAccelGroup *create_menus(void)
     ADD_MENU_ITEM(N_("Open..."), menu_open, GDK_o, 0);
     SEPARATOR;                  /* ------------------------------------ */
     ADD_MENU_ITEM(N_("Load collection..."), deserialize_collection, 0, 0);
-    ADD_MENU_ITEM(N_("Save collection..."), serialize_collection, 0, 0);
+    ADD_MENU_ITEM(N_("Save collection..."), serialize_collection_gui, 0, 0);
     SEPARATOR;                  /* ------------------------------------ */
     ADD_MENU_ITEM(N_("Quit"), gui_quit, GDK_q, 0);
 
@@ -377,15 +378,16 @@ GtkAccelGroup *create_menus(void)
     SEPARATOR;                  /* ------------------------------------ */
     ADD_MENU_ITEM(N_("Load previous image"), load_direction, GDK_p, -1);
     ADD_MENU_ITEM(N_("Load next image"), load_direction, GDK_n, 1);
+    ADD_MENU_ITEM(N_("Load the Nth image..."), menu_image_nr, GDK_g, 0);
+    SEPARATOR;                  /* ------------------------------------ */
+    ADD_MENU_ITEM(N_("Sort images list"), reorder_files, 0, FALSE);
+    ADD_MENU_ITEM(N_("Shuffle images list"), reorder_files, 0, TRUE);
     SEPARATOR;                  /* ------------------------------------ */
     ADD_MENU_ITEM(N_("Rebuild images menus"), rebuild_images_menus, 0, 0);
-    ADD_MENU_ITEM(N_("Sort images list"), reorder_files, 0, FALSE);
     item = ADD_MENU_ITEM(N_("Stop rebuilding the images menu"),
                          cancel_using_tree, 0, 0);
     set_stop_rebuilding_menu(item);
-
-    ADD_MENU_ITEM(N_("Shuffle images list"), reorder_files, 0, TRUE);
-
+    SEPARATOR;                  /* ------------------------------------ */
     item = ADD_MENU_ITEM(N_("Start the slide show"), toggle_slide_show, 0,
                          FALSE);
     set_slide_show_menu(item);
@@ -419,10 +421,10 @@ GtkAccelGroup *create_menus(void)
 
     SEPARATOR;                  /* ------------------------------------ */
 
-    ADD_MENU_ITEM(N_("Rotate +45 deg"), menu_transformation, 0,
+    ADD_MENU_ITEM(N_("Rotate +90 deg"), menu_transformation, 0,
                   TRANSFORMATION_ROTATE_BIG_PLUS);
 
-    ADD_MENU_ITEM(N_("Rotate -45 deg"), menu_transformation, 0,
+    ADD_MENU_ITEM(N_("Rotate -90 deg"), menu_transformation, 0,
                   TRANSFORMATION_ROTATE_BIG_MINUS);
 
     ADD_MENU_ITEM(N_("Rotate +0.1 deg"), menu_transformation, 0,
