@@ -22,6 +22,12 @@
  * Which image to load next *
  ****************************/
 
+#include <sys/types.h>          /* off_t */
+#include <sys/stat.h>           /* struct stat, stat() */
+#include <time.h>               /* time_t */
+#include <stdio.h>              /* perror() */
+#include <string.h>             /* memcmp() */
+
 #include "gliv.h"
 #include "next_image.h"
 #include "options.h"
@@ -31,7 +37,6 @@
 #include "files_list.h"
 #include "textures.h"
 #include "messages.h"
-#include "main.h"
 #include "menus.h"
 #include "callbacks.h"
 #include "transition.h"
@@ -50,6 +55,15 @@ static GlivImage *next_image = NULL;
 
 static GtkMenuItem *slide_show_menu;
 static guint slide_show_timer = 0;
+static gint slide_show_direction = 1;
+
+static void forget_image(GlivImage ** im_ptr)
+{
+    if (*im_ptr != NULL) {
+        g_object_unref(*im_ptr);
+        *im_ptr = NULL;
+    }
+}
 
 /*
  * Once the images list has been reordered we must take care
@@ -61,13 +75,11 @@ void after_reorder(gint new_id)
     if (previous_image != NULL &&
         current_image->node->prev != previous_image->node) {
 
-        g_object_unref(previous_image);
-        previous_image = NULL;
+        forget_image(&previous_image);
     }
 
     if (next_image != NULL && current_image->node->next != next_image->node) {
-        g_object_unref(next_image);
-        next_image = NULL;
+        forget_image(&next_image);
     }
 
     current_image->number = new_id;
@@ -143,6 +155,8 @@ const gchar *get_image_notice(void)
 
 /*** Switching images ***/
 
+static void slide_show_advance(void);
+
 static void render_next_image(GlivImage * im)
 {
     gfloat *old_matrix;
@@ -169,6 +183,9 @@ static void render_next_image(GlivImage * im)
 
     else if (im->node->prev == NULL && first_image == FALSE)
         image_notice(FALSE);
+
+    if (slide_show_started())
+        slide_show_advance();
 }
 
 /*** Next/previous image ***/
@@ -261,13 +278,8 @@ static void destroy_unused(GlivImage ** backup)
         *backup = NULL;
     }
 
-    if (options->one_image) {
-        g_object_unref(current_image);
-        current_image = NULL;
-    } else {
-        *backup = current_image;
-        g_object_ref(*backup);
-    }
+    *backup = current_image;
+    g_object_ref(*backup);
 }
 
 /* Return FALSE if there are no more images. */
@@ -343,9 +355,14 @@ gboolean load_direction(gint dir)
 
         process_events();
 
-        if (options->one_image == FALSE)
+        if (options->one_image) {
+            g_object_unref(*backup);
+            *backup = NULL;
+        } else
             *next = load_in_direction(dir);
 
+        slide_show_direction = dir;
+
         /* Usually next_direction == 0, and the while loop ends. */
         dir = next_direction;
     }
@@ -368,10 +385,7 @@ void open_next_image(gboolean keep_next)
         return;
 
     if (keep_next) {
-        if (previous_image != NULL) {
-            g_object_unref(previous_image);
-            previous_image = NULL;
-        }
+        forget_image(&previous_image);
     } else
         unload_images();
 
@@ -422,106 +436,108 @@ void load_second_image(void)
         start_slide_show();
 }
 
-/*** Random loading ***/
-
-void open_file(const gchar * name, gboolean add_dir, gboolean shuffle_flag)
-{
-    gint nb_inserted;
-
-    if (is_loading())
-        return;
-
-    if (g_file_test(name, G_FILE_TEST_IS_DIR) || add_dir == FALSE)
-        nb_inserted = insert_after_current((gchar **) & name, 1, FALSE,
-                                           shuffle_flag, !shuffle_flag, TRUE);
-
-    else {
-        gchar *dir;
-
-        if (options->force == FALSE) {
-            switch (get_loader(name)) {
-            case LOADER_NONE:
-            case LOADER_DOT_GLIV:
-            case LOADER_DECOMP_DOT_GLIV:
-                return;
-
-            default:
-                /* To avoid a gcc warning. */
-                break;
-            }
-        }
-
-        /* Add all files in the directory in the list. */
-        dir = g_path_get_dirname(name);
-
-        nb_inserted = insert_after_current(&dir, 1, FALSE,
-                                           shuffle_flag, !shuffle_flag, TRUE);
-        g_free(dir);
-
-        /* Put the requested image at current+1 position in the file list. */
-        if (nb_inserted > 0)
-            place_next(name);
-    }
-
-    if (nb_inserted > 0)
-        open_next_image(nb_inserted == 1);
-}
-
 /*
  * Reloading.
  * Used for example, when an option is changed such as dithering or mipmap.
  */
 
-static GlivImage *reload_image(GlivImage * im)
-{
-    GlivImage *new;
-
-    new = load_file(im->node->data);
-    if (new == NULL) {
-        g_object_ref(im);
-        return im;
-    }
-
-    new->node = im->node;
-    new->number = im->number;
-
-    return new;
-}
-
-void reload_all_images(void)
+static void reload_current_image(void)
 {
     GlivImage *new;
 
     if (is_loading() || current_image == NULL)
         return;
 
-    new = reload_image(current_image);
+    new = load_file(current_image->node->data);
+    if (new == NULL)
+        return;
+
+    new->node = current_image->node;
+    new->number = current_image->number;
+
     g_object_unref(current_image);
-    current_image = new;
-    prioritize_textures(current_image, TRUE);
 
-    refresh(REFRESH_NOW);
+    if (current_image != new) {
+        current_image = new;
 
-    if (previous_image == next_image && previous_image != NULL) {
-        new = reload_image(previous_image);
-        g_object_unref(previous_image);
-        g_object_unref(next_image);
-        previous_image = next_image = new;
-    } else {
-        if (previous_image != NULL) {
-            new = reload_image(previous_image);
-            g_object_unref(previous_image);
-            previous_image = new;
-        }
-
-        if (next_image != NULL) {
-            new = reload_image(next_image);
-            g_object_unref(next_image);
-            next_image = new;
-        }
+        prioritize_textures(current_image, TRUE);
+        refresh(REFRESH_NOW);
     }
 }
 
+void reload_images(void)
+{
+    if (is_loading())
+        return;
+
+    forget_image(&previous_image);
+    forget_image(&next_image);
+    reload_current_image();
+}
+
+struct image_stat {
+    off_t size;
+    time_t ctime;
+    gboolean present;
+};
+
+struct image_stat3 {
+    struct image_stat previous;
+    struct image_stat current;
+    struct image_stat next;
+};
+
+static void stat_image(GlivImage * im, struct image_stat *st)
+{
+    st->present = FALSE;
+
+    if (im != NULL) {
+        struct stat real_stat;
+
+        if (stat(im->node->data, &real_stat) < 0) {
+            perror(im->node->data);
+            return;
+        }
+
+        st->size = real_stat.st_size;
+        st->ctime = real_stat.st_ctime;
+        st->present = TRUE;
+    }
+}
+
+gpointer stat_loaded_files(void)
+{
+    struct image_stat3 *stat_data = g_new(struct image_stat3, 1);
+
+    stat_image(previous_image, &stat_data->previous);
+    stat_image(current_image, &stat_data->current);
+    stat_image(next_image, &stat_data->next);
+
+    return stat_data;
+}
+
+#define DIFFERENT(name) \
+    memcmp(&before->name, &now->name, sizeof(struct image_stat))
+
+void reload_changed_files(gpointer * stat_data)
+{
+    struct image_stat3 *before = (struct image_stat3 *) stat_data;
+    struct image_stat3 *now = stat_loaded_files();
+
+    if (DIFFERENT(previous))
+        forget_image(&previous_image);
+
+    if (DIFFERENT(next))
+        forget_image(&next_image);
+
+    if (DIFFERENT(current))
+        reload_current_image();
+
+    g_free(before);
+    g_free(now);
+}
+
+
 /*** Menu ***/
 
 /* Check if we can optimize the loading asked by the images menu. */
@@ -669,6 +685,14 @@ void unload_images(void)
 
 /*** Slide show ***/
 
+static void slide_show_remove_timer(void)
+{
+    if (slide_show_timer != 0) {
+        g_source_remove(slide_show_timer);
+        slide_show_timer = 0;
+    }
+}
+
 void set_slide_show_menu(GtkMenuItem * item)
 {
     slide_show_menu = item;
@@ -676,35 +700,36 @@ void set_slide_show_menu(GtkMenuItem * item)
 
 static void stop_slide_show(void)
 {
-    if (slide_show_started()) {
-        g_source_remove(slide_show_timer);
-        slide_show_timer = 0;
-    }
-
+    slide_show_remove_timer();
     set_menu_label(slide_show_menu, _("Start the slide show"), TRUE);
 }
 
 static gboolean slide_show_next(void)
 {
-    if (load_direction(1) == FALSE) {
+    if (load_direction(slide_show_direction) == FALSE) {
         /* No more images. */
         stop_slide_show();
-        return FALSE;
     }
 
-    return TRUE;
+    return FALSE;
+}
+
+static void slide_show_advance(void)
+{
+    slide_show_remove_timer();
+    slide_show_timer = g_timeout_add(options->duration * 1000,
+                                     (GSourceFunc) slide_show_next, NULL);
 }
 
 void start_slide_show(void)
 {
     stop_slide_show();
+    slide_show_direction = 1;
 
     if (options->duration < 0 || current_image == NULL)
         return;
 
-    slide_show_timer = g_timeout_add(options->duration * 1000,
-                                     (GSourceFunc) slide_show_next, NULL);
-
+    slide_show_advance();
     set_menu_label(slide_show_menu, _("Stop the slide show"), TRUE);
 }
 
