@@ -99,7 +99,7 @@ static void apply(void)
         options->dither = new_options->dither;
         options->mipmap = new_options->mipmap;
 
-        reload_all_images();
+        reload_images();
         what |= REFRESH_IMAGE;
     }
 
@@ -119,11 +119,11 @@ static void apply(void)
         }
 
     if (is_rebuilding_a_menu() == FALSE)
-	if (options->thumbnails != new_options->thumbnails ||
-	    options->thumb_width != new_options->thumb_width ||
-	    options->thumb_height != new_options->thumb_height ||
-	    options->mnemonics != new_options->mnemonics)
-	    invalidate_last_tree();
+        if (options->thumbnails != new_options->thumbnails ||
+            options->thumb_width != new_options->thumb_width ||
+            options->thumb_height != new_options->thumb_height ||
+            options->mnemonics != new_options->mnemonics)
+            invalidate_last_tree();
 
     bg_color_changed = !gdk_color_equal(&options->bg_col, &new_options->bg_col);
 
@@ -197,10 +197,44 @@ static gboolean on_save_now_clicked(void)
     return FALSE;
 }
 
+static gboolean on_read_config_realize(GtkLabel * widget)
+{
+    const gchar *filename = get_read_config_file();
+
+    if (filename == NULL)
+        filename = _("NONE");
+
+    gchar *text =
+        g_strdup_printf(_("This configuration file has been read: %s"),
+                        filename);
+
+    gtk_label_set_text(widget, text);
+    g_free(text);
+
+    return FALSE;
+}
+
+static gboolean on_write_config_realize(GtkLabel * widget)
+{
+    const gchar *filename = get_write_config_file();
+
+    if (filename == NULL)
+        filename = _("NONE");
+
+    gchar *text =
+        g_strdup_printf(_("This configuration file will be written: %s"),
+                        filename);
+
+    gtk_label_set_text(widget, text);
+    g_free(text);
+
+    return FALSE;
+}
+
 /* Transitions */
 
 static gboolean on_transitions_toggled(GtkTable * transitions_table,
-				       GtkToggleButton * button)
+                                       GtkToggleButton * button)
 {
     gboolean activated = gtk_toggle_button_get_active(button);
 
@@ -208,7 +242,8 @@ static gboolean on_transitions_toggled(GtkTable * transitions_table,
     return FALSE;
 }
 
-static gboolean on_transition_table_realize(GtkWidget * transitions_table) {
+static gboolean on_transition_table_realize(GtkWidget * transitions_table)
+{
     gtk_widget_set_sensitive(transitions_table, options->transitions);
     return FALSE;
 }
@@ -224,9 +259,10 @@ gboolean show_options(void)
     options_dialog = GTK_DIALOG(create_options_dialog());
 
     response = run_modal_dialog(options_dialog);
+    gtk_widget_destroy(GTK_WIDGET(options_dialog));
+
     if (response == GTK_RESPONSE_OK)
         apply();
 
-    gtk_widget_destroy(GTK_WIDGET(options_dialog));
     return FALSE;
 }
