@@ -33,14 +33,12 @@
 #include "params.h"
 #include "zoom_frame.h"
 #include "scrollbars.h"
-#include "gl_widget.h"
 #include "textures.h"
 #include "files_list.h"
 #include "loading.h"
 #include "windows.h"
 #include "history.h"
 #include "next_image.h"
-#include "messages.h"
 #include "opengl.h"
 #include "callbacks.h"
 #include "transition.h"
@@ -326,7 +324,7 @@ gfloat *prepare_rendering(GlivImage * im)
     im->ident = NULL;
 
     if (options->fullscreen == FALSE)
-        goto_window(im);
+        goto_window(im, FALSE);
 
     /* Be sure to update the display now. A transition may take place ... */
     gtk_widget_queue_draw(GTK_WIDGET(get_current_window()));
@@ -417,7 +415,8 @@ void refresh(gint what)
                 refresh(REFRESH_IMAGE);
 
             else {
-                gint diff = G_USEC_PER_SEC / options->fps - G_USEC_PER_SEC / fps;
+                gint diff =
+                    G_USEC_PER_SEC / options->fps - G_USEC_PER_SEC / fps;
                 diff /= 1000;
 
                 if (diff > 0)
