@@ -23,9 +23,8 @@
  *****************************/
 
 #include <gtk/gtkgl.h>
-#include <sys/time.h>           /* struct timeval, gettimeofday() */
-#include <time.h>               /* struct timespec, nanosleep() */
 #include <string.h>             /* memcpy() */
+#include <stdio.h>              /* printf(), puts() */
 
 #include "gliv.h"
 #include "transition.h"
@@ -35,7 +34,6 @@
 #include "rendering.h"
 #include "options.h"
 #include "loading.h"
-#include "gl_widget.h"
 #include "callbacks.h"
 #include "messages.h"
 
@@ -78,33 +76,32 @@ static void draw_image(GlivImage * im, gfloat * matrix, gfloat opacity)
 }
 
 /* res = big - small */
-static void diff_timeval(struct timeval *res,
-                         struct timeval *big, struct timeval *small)
+static void diff_timeval(GTimeVal * res, GTimeVal * big, GTimeVal * small)
 {
     res->tv_sec = big->tv_sec - small->tv_sec;
     if (big->tv_usec >= small->tv_usec)
         res->tv_usec = big->tv_usec - small->tv_usec;
     else {
         res->tv_sec--;
-        res->tv_usec = big->tv_usec + 1000000 - small->tv_usec;
+        res->tv_usec = big->tv_usec + G_USEC_PER_SEC - small->tv_usec;
     }
 }
 
-static void sleep_remaining(struct timeval *started, struct timeval *required)
+static void sleep_remaining(GTimeVal * started, GTimeVal * required)
 {
-    struct timeval now;
-    struct timeval done;
-    struct timeval tv_sleep_time;
-    struct timespec ts_sleep_time;
+    GTimeVal now;
+    GTimeVal done;
+    GTimeVal sleep_time;
+    glong delay;
 
-    gettimeofday(&now, NULL);
+    g_get_current_time(&now);
 
     diff_timeval(&done, &now, started);
-    diff_timeval(&tv_sleep_time, required, &done);
+    diff_timeval(&sleep_time, required, &done);
 
-    ts_sleep_time.tv_sec = tv_sleep_time.tv_sec;
-    ts_sleep_time.tv_nsec = tv_sleep_time.tv_usec * 1000;
-    nanosleep(&ts_sleep_time, NULL);
+    delay = sleep_time.tv_sec * G_USEC_PER_SEC + sleep_time.tv_usec;
+    if (delay > 0)
+        g_usleep(delay);
 }
 
 
@@ -118,8 +115,8 @@ static void transition_no_alpha_black(GlivImage * im1, gfloat * matrix1,
 {
     gint sleep, step;
     gfloat alpha;
-    struct timeval sleeping_time;
-    struct timeval now;
+    GTimeVal sleeping_time;
+    GTimeVal now;
     GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable(gl_widget);
 
     glEnable(GL_BLEND);
@@ -140,7 +137,7 @@ static void transition_no_alpha_black(GlivImage * im1, gfloat * matrix1,
         draw_image(im1, matrix1, 1.0 - alpha);
         draw_image(im2, matrix2, alpha);
 
-        gettimeofday(&now, NULL);
+        g_get_current_time(&now);
 
         gdk_gl_drawable_swap_buffers(gldrawable);
         gdk_gl_drawable_wait_gl(gldrawable);
@@ -167,6 +164,7 @@ static GlivImage *grab_image(GlivImage * im, gfloat * matrix,
     gint viewport[4], x, y, w, h;
     GdkPixbuf *pixbuf;
     gboolean old_mipmap;
+    gboolean old_dither;
 
     *new_mat = new_matrix();
 
@@ -202,23 +200,26 @@ static GlivImage *grab_image(GlivImage * im, gfloat * matrix,
 
 #if 0
     {
-	static gint id = 0;
-	gchar * base = g_path_get_basename(im->node->data);
-	gchar * filename = g_strdup_printf("%s_%d.png", base, id++);
+        static gint id = 0;
+        gchar *base = g_path_get_basename(im->node->data);
+        gchar *filename = g_strdup_printf("%s_%d.png", base, id++);
 
-	g_free(base);
-	gdk_pixbuf_save(pixbuf, filename, "png", NULL, NULL);
-	g_free(filename);
+        g_free(base);
+        gdk_pixbuf_save(pixbuf, filename, "png", NULL, NULL);
+        g_free(filename);
     }
 #endif
 
-    /* We don't need mipmaps for this image */
+    /* We don't need mipmaps and dithering for this image */
     old_mipmap = options->mipmap;
+    old_dither = options->dither;
     options->mipmap = FALSE;
+    options->dither = FALSE;
 
     new_im = make_gliv_image(pixbuf);
 
     options->mipmap = old_mipmap;
+    options->dither = old_dither;
 
     return new_im;
 }
@@ -268,6 +269,17 @@ void transition(gfloat * old_matrix, GlivImage * im)
         gfloat *matrix1, *matrix2;
         gfloat *matrix2_tmp = get_matrix_for_image(im);
 
+        printf("Transition slow case: (cur: %s, im: %s, bg: %s)\n",
+               current_image->has_alpha ? "alpha" : "no alpha",
+               im->has_alpha ? "alpha" : "no alpha",
+               bg_is_black ? "black" : "not black");
+
+        if (options->one_image) {
+            puts("The \"Only one image mode\" is incompatible "
+                 "with slow transitions");
+            goto out;
+        }
+
         im1 = grab_image(current_image, old_matrix, &matrix1, bg_is_black);
         im2 = grab_image(im, matrix2_tmp, &matrix2, bg_is_black);
 
@@ -289,7 +301,7 @@ void transition(gfloat * old_matrix, GlivImage * im)
         g_object_unref(im1);
         g_object_unref(im2);
     }
-
+  out:
     in_transition = FALSE;
     set_loading(NULL);
 }
